@@ -15,4 +15,13 @@ public class FilteredReadsMetric extends MetricBase {
 		this.READS_ACCEPTED=0;
 		this.READS_REJECTED=0;
 	}
+
+	public FilteredReadsMetric copy () {
+		return new FilteredReadsMetric();
+	}
+
+	public void merge(final FilteredReadsMetric other) {
+		this.READS_ACCEPTED += other.READS_ACCEPTED;
+		this.READS_REJECTED += other.READS_REJECTED;
+	}
 }
